#ifndef __CDOS_AD_AUTH_H__
#define __CDOS_AD_AUTH_H__

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef ZERO_STRUCT
#define ZERO_STRUCT(x) memset((char *)&(x), 0, sizeof(x))
#else
#undef ZERO_STRUCT
#define ZERO_STRUCT(x) memset((char *)&(x), 0, sizeof(x))
#endif


#define WINBIND_SUCCESS 0		/* Successful function return */
#define WINBIND_OPEN_ERR 1		/* dlopen() failure when dynamically */
				/* loading a service module */
#define WINBIND_SYMBOL_ERR 2	/* Symbol not found */
#define WINBIND_SERVICE_ERR 3	/* Error in service module */
#define WINBIND_SYSTEM_ERR 4	/* System error */
#define WINBIND_BUF_ERR 5		/* Memory buffer error */
#define WINBIND_PERM_DENIED 6	/* Permission denied */
#define WINBIND_AUTH_ERR 7		/* Authentication failure */
#define WINBIND_CRED_INSUFFICIENT 8	/* Can not access authentication data */
				/* due to insufficient credentials */
#define WINBIND_AUTHINFO_UNAVAIL 9	/* Underlying authentication service */
				/* can not retrieve authentication */
				/* information  */
#define WINBIND_USER_UNKNOWN 10	/* User not known to the underlying */
				/* authenticaiton module */
#define WINBIND_MAXTRIES 11		/* An authentication service has */
				/* maintained a retry count which has */
				/* been reached.  No further retries */
				/* should be attempted */
#define WINBIND_NEW_AUTHTOK_REQD 12	/* New authentication token required. */
				/* This is normally returned if the */
				/* machine security policies require */
				/* that the password should be changed */
				/* beccause the password is NULL or it */
				/* has aged */
#define WINBIND_ACCT_EXPIRED 13	/* User account has expired */
#define WINBIND_SESSION_ERR 14	/* Can not make/remove an entry for */
				/* the specified session */
#define WINBIND_CRED_UNAVAIL 15	/* Underlying authentication service */
				/* can not retrieve user credentials */
                                /* unavailable */
#define WINBIND_CRED_EXPIRED 16	/* User credentials expired */
#define WINBIND_CRED_ERR 17		/* Failure setting user credentials */
#define WINBIND_NO_MODULE_DATA 18	/* No module specific data is present */
#define WINBIND_CONV_ERR 19		/* Conversation error */
#define WINBIND_AUTHTOK_ERR 20	/* Authentication token manipulation error */
#define WINBIND_AUTHTOK_RECOVERY_ERR 21 /* Authentication information */
				    /* cannot be recovered */
#define WINBIND_AUTHTOK_LOCK_BUSY 22   /* Authentication token lock busy */
#define WINBIND_AUTHTOK_DISABLE_AGING 23 /* Authentication token aging disabled */
#define WINBIND_TRY_AGAIN 24	/* Preliminary check by password service */
#define WINBIND_IGNORE 25		/* Ignore underlying account module */
				/* regardless of whether the control */
				/* flag is required, optional, or sufficient */
#define WINBIND_ABORT 26            /* Critical error (?module fail now request) */
#define WINBIND_AUTHTOK_EXPIRED  27 /* user's authentication token has expired */
#define WINBIND_MODULE_UNKNOWN   28 /* module is not known */

#define WINBIND_BAD_ITEM         29 /* Bad item passed to pam_*_item() */
#define WINBIND_CONV_AGAIN       30 /* conversation function is event driven
				     and data is not available yet */
#define WINBIND_INCOMPLETE       31 /* please call this function again to
				   complete authentication stack. Before
				   calling again, verify that conversation
				   is completed */


enum passwordChangeRejectReason {
	PWD_CHANGE_NO_ERROR=0,
	PWD_CHANGE_PASSWORD_TOO_SHORT=1,
	PWD_CHANGE_PWD_IN_HISTORY=2,
	PWD_CHANGE_USERNAME_IN_PASSWORD=3,
	PWD_CHANGE_FULLNAME_IN_PASSWORD=4,
	PWD_CHANGE_NOT_COMPLEX=5,
	PWD_CHANGE_MACHINE_NOT_DEFAULT=6,
	PWD_CHANGE_FAILED_BY_FILTER=7,
	PWD_CHANGE_PASSWORD_TOO_LONG=8
};

struct policy_info
{
    uint32_t min_length;  // 密码最短长度
    uint32_t history;  // 密码保存历史
    uint32_t properties;  // 密码复杂度
    uint64_t expire;  // 密码过期时间
    uint64_t age;  // 密码最短使用时间
};

struct change_info
{
    int status;  //返回状态
    enum passwordChangeRejectReason reject_code;
    char *message;  //错误或警告信息
    struct policy_info *policy;  //密码策略
};

typedef struct change_info changeInfo;


struct auth_info
{
    int status;  //返回状态 0成功
    char *message;  //错误或警告信息
    uint16_t bad_password_count;  //密码错误次数
    uint64_t logon_time;  //登录时间
    uint64_t pass_last_set_time;  //密码最后修改时间
    uint64_t pass_can_change_time;  //密码能够被修改时间
    uint64_t pass_must_change_time;  //密码必须要修改的时间
};

typedef struct auth_info authInfo;

void destroy_changeInfo(changeInfo *);
void destroy_authInfo(authInfo *);

// 必须使用destroy_changeInfo释放内存
changeInfo *set_password(const char *user,
                         const char *old_password,
                         const char *new_password);

// 必须使用destroy_authInfo释放内存
authInfo *authenticate(char *user, char *password);

int authenticate_pam(char *user, char *password);

int get_expired_days(authInfo *);

#ifdef __cplusplus
}
#endif

#endif /*__CDOS_AD_AUTH_H__*/
